@@ -1,11 +1,10 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2018-2022, by Samuel Williams.
+# Copyright, 2018-2024, by Samuel Williams.
 
-require 'async/clock'
-
-require 'timer_quantum'
+require "async/clock"
+require "sus/fixtures/time/quantum"
 
 describe Async::Clock do
 	let(:clock) {subject.new}
@@ -15,7 +14,7 @@ describe Async::Clock do
 			sleep 0.01
 		end
 		
-		expect(duration).to be_within(Q).of(0.01)
+		expect(duration).to be_within(Sus::Fixtures::Time::QUANTUM).of(0.01)
 	end
 	
 	it "can get current offset" do
@@ -29,11 +28,11 @@ describe Async::Clock do
 			clock.stop!
 		end
 		
-		expect(clock.total).to be_within(2 * Q).of(0.02)
+		expect(clock.total).to be_within(2 * Sus::Fixtures::Time::QUANTUM).of(0.02)
 	end
 	
-	with '#total' do
-		with 'initial duration' do
+	with "#total" do
+		with "initial duration" do
 			let(:clock) {subject.new(1.5)}
 			let(:total) {clock.total}
 			
@@ -51,7 +50,7 @@ describe Async::Clock do
 		end
 	end
 	
-	with '.start' do
+	with ".start" do
 		let(:clock) {subject.start}
 		let(:total) {clock.total}
 		
@@ -59,4 +58,22 @@ describe Async::Clock do
 			expect(total).to be >= 0.0
 		end
 	end
+	
+	with "#reset!" do
+		it "resets the total time" do
+			clock.start!
+			sleep(0.0001)
+			clock.stop!
+			expect(clock.total).to be > 0.0
+			clock.reset!
+			expect(clock.total).to be == 0.0
+		end
+		
+		it "resets the start time" do
+			clock.start!
+			clock.reset!
+			sleep(0.0001)
+			expect(clock.total).to be > 0.0
+		end
+	end
 end
