@@ -5,21 +5,22 @@
 # Copyright, 2023, by Math Ieu.
 # Copyright, 2023, by Emil Tin.
 
-require 'async'
-require 'async/clock'
-require 'async/queue'
+require "async"
+require "async/clock"
+require "async/queue"
 
-require 'timer_quantum'
+require "sus/fixtures/console"
+
+require "sus/fixtures/time/quantum"
 
 describe Async::Task do
 	let(:reactor) {Async::Reactor.new}
 	
-	def after
-		reactor.close
-		super
+	after do
+		Fiber.set_scheduler(nil)
 	end
 	
-	with '#annotate' do
+	with "#annotate" do
 		it "can annotate the current task that has not started yet" do
 			task = Async::Task.new(reactor) do |task|
 				sleep
@@ -45,11 +46,11 @@ describe Async::Task do
 		end
 	end
 	
-	with '.yield' do
+	with ".yield" do
 		it "can yield back to scheduler" do
 			state = nil
 			
-			reactor.async do |task|
+			reactor.run do |task|
 				child = task.async do
 					state = :yielding
 					Async::Task.yield
@@ -59,13 +60,11 @@ describe Async::Task do
 				Fiber.scheduler.resume(child.fiber)
 			end
 			
-			reactor.run
-			
 			expect(state).to be == :yielded
 		end
 	end
 	
-	with '#run' do
+	with "#run" do
 		it "can't be invoked twice" do
 			task = reactor.async do |task|
 			end
@@ -74,22 +73,22 @@ describe Async::Task do
 		end
 	end
 	
-	with '#current?' do
+	with "#current?" do
 		it "can check if it is the currently running task" do
 			task = reactor.async do |task|
 				expect(task).to be(:current?)
-				task.sleep(0.1)
+				sleep(0.1)
 			end
 			
 			expect(task).not.to be(:current?)
 		end
 	end
 	
-	with '#async' do
+	with "#async" do
 		it "can start child async tasks" do
 			parent = reactor.async do |task|
 				child = task.async do
-					task.sleep(1)
+					sleep(1)
 				end
 				
 				expect(child).not.to be_nil
@@ -125,6 +124,8 @@ describe Async::Task do
 			reactor.run do
 				expect do
 					reactor.async do |task|
+						expect(task).to receive(:warn).and_return(nil)
+						
 						raise "boom"
 					end.wait
 				end.to raise_exception(RuntimeError, message: be =~ /boom/)
@@ -155,6 +156,8 @@ describe Async::Task do
 			
 			expect do
 				task = reactor.async do |task|
+					expect(task).to receive(:warn).and_return(nil)
+					
 					raise "boom"
 				end
 			end.not.to raise_exception
@@ -189,7 +192,7 @@ describe Async::Task do
 		end
 	end
 	
-	with '#yield' do
+	with "#yield" do
 		it "can yield back to reactor" do
 			state = nil
 			
@@ -205,7 +208,7 @@ describe Async::Task do
 		end
 	end
 	
-	with '#stop' do
+	with "#stop" do
 		it "can't stop finished tasks" do
 			task = reactor.async{}
 			
@@ -259,7 +262,7 @@ describe Async::Task do
 			reactor.run do
 				task = reactor.async do |task|
 					state = :started
-					task.sleep(10)
+					sleep(10)
 					state = :finished
 				end
 				
@@ -275,7 +278,7 @@ describe Async::Task do
 			reactor.run do
 				task = reactor.async do |task|
 					child = task.async do |subtask|
-						subtask.sleep(1)
+						sleep(1)
 					end
 					
 					begin
@@ -364,10 +367,10 @@ describe Async::Task do
 					task.async do |task|
 						child_task = task
 						
-						task.sleep(10)
+						sleep(10)
 					end
 					
-					task.sleep(10)
+					sleep(10)
 				end
 				
 				expect(parent_task).not.to be_nil
@@ -399,7 +402,7 @@ describe Async::Task do
 						end
 					end.resume
 					
-					task.sleep(1)
+					sleep(1)
 				end
 			end
 			
@@ -416,21 +419,21 @@ describe Async::Task do
 					
 					task.async do |task|
 						children_tasks << task
-						task.sleep(0.02)
+						sleep(0.02)
 					end
 					
 					task.async do |task|
 						children_tasks << task
-						task.sleep(0.01)
+						sleep(0.01)
 						parent_task.stop
 					end
 					
 					task.async do |task|
 						children_tasks << task
-						task.sleep(0.02)
+						sleep(0.02)
 					end
 					
-					task.sleep(0.02)
+					sleep(0.02)
 				end
 			end
 			
@@ -480,11 +483,11 @@ describe Async::Task do
 							
 							ready.enqueue(true)
 							
-							task.sleep(10)
+							sleep(10)
 						end
-						task.sleep(10)
+						sleep(10)
 					end
-					task.sleep(10)
+					sleep(10)
 				end
 				
 				ready.dequeue
@@ -540,14 +543,14 @@ describe Async::Task do
 		end
 	end
 	
-	with '#sleep' do
+	with "#sleep" do
 		let(:duration) {0.01}
 		
 		it "can sleep for the requested duration" do
 			state = nil
 			
 			reactor.async do |task|
-				task.sleep(duration)
+				sleep(duration)
 				state = :finished
 			end
 			
@@ -556,36 +559,20 @@ describe Async::Task do
 			end
 			
 			# This is too unstable on travis.
-			expect(time).to be_within(Q).of(duration)
+			expect(time).to be_within(Sus::Fixtures::Time::QUANTUM).of(duration)
 			expect(state).to be == :finished
 		end
 	end
 	
-	with '#with_timeout' do
-		it "can extend timeout" do
-			reactor.async do |task|
-				task.with_timeout(0.02) do |timer|
-					task.sleep(0.01)
-					
-					expect(timer.fires_in).to be_within(Q).of(0.01)
-					
-					timer.reset
-					
-					expect(timer.fires_in).to be_within(Q).of(0.02)
-				end
-			end
-			
-			reactor.run
-		end
-		
+	with "#with_timeout" do
 		it "will timeout if execution takes too long" do
 			state = nil
 			
-			reactor.async do |task|
+			Sync do |task|
 				begin
 					task.with_timeout(0.01) do
 						state = :started
-						task.sleep(10)
+						sleep(10)
 						state = :finished
 					end
 				rescue Async::TimeoutError
@@ -593,8 +580,6 @@ describe Async::Task do
 				end
 			end
 			
-			reactor.run
-			
 			expect(state).to be == :timeout
 		end
 		
@@ -602,17 +587,15 @@ describe Async::Task do
 			input, output = IO.pipe
 			error = nil
 			
-			reactor.async do |task|
+			Sync do |task|
 				begin
 					# This can invoke `io_wait`, which previously had `rescue TimeoutError`, causing the timeout to be ignored.
 					task.with_timeout(0.1) {input.gets}
 				rescue Async::TimeoutError => error
-				  # Ignore.
+					# Ignore.
 				end
 			end
 			
-			reactor.run
-			
 			expect(error).to be_a(Async::TimeoutError)
 		ensure
 			input.close
@@ -625,7 +608,7 @@ describe Async::Task do
 			reactor.async do |task|
 				state = :started
 				task.with_timeout(0.01) do
-					task.sleep(0.001)
+					sleep(0.001)
 					state = :finished
 				end
 			end
@@ -637,12 +620,16 @@ describe Async::Task do
 		
 		def sleep_forever
 			while true
-				Async::Task.current.sleep(1)
+				sleep(1)
 			end
 		end
 		
 		it "contains useful backtrace" do
 			task = Async do |task|
+				expect(task).to receive(:warn).with_options(have_keys(
+					exception: be_a(Async::TimeoutError),
+				)).and_return(nil)
+				
 				task.with_timeout(0.001) do
 					sleep_forever
 				end
@@ -655,11 +642,11 @@ describe Async::Task do
 		end
 	end
 	
-	with '#backtrace' do
+	with "#backtrace" do
 		it "has a backtrace" do
 			Async do
 				task = Async do |task|
-					task.sleep(1)
+					sleep(1)
 				end
 				
 				expect(task.backtrace).to have_value(be =~ /sleep/)
@@ -677,16 +664,16 @@ describe Async::Task do
 		end
 	end
 	
-	with '#wait' do
+	with "#wait" do
 		it "will wait on another task to complete" do
 			apples_task = reactor.async do |task|
-				task.sleep(0.01)
+				sleep(0.01)
 				
 				:apples
 			end
 			
 			oranges_task = reactor.async do |task|
-				task.sleep(0.01)
+				sleep(0.01)
 				
 				:oranges
 			end
@@ -705,6 +692,8 @@ describe Async::Task do
 			
 			reactor.run do
 				error_task = reactor.async do |task|
+					expect(task).to receive(:warn).and_return(nil)
+					
 					raise RuntimeError, "brain not provided"
 				end
 				
@@ -743,25 +732,30 @@ describe Async::Task do
 		end
 	end
 	
-	with '#result' do
-		it 'does not raise exception' do
-			reactor.async do
-				task = reactor.async do
-					raise "The space time converter has failed."
-				end
+	with "#result" do
+		it "does not raise exception" do
+			task = reactor.async do |task|
+				expect(task).to receive(:warn).and_return(nil)
 				
-				expect(task.result).to be_a(RuntimeError)
+				raise "The space time converter has failed."
 			end
+			
+			reactor.run do
+				expect do
+					task.wait
+				end.to raise_exception(RuntimeError, message: be =~ /space time converter/)
+			end
+			
+			expect(task.result).to be_a(RuntimeError)
 		end
 		
-		it 'does not wait for task completion' do
+		it "does not wait for task completion" do
 			task = reactor.async do |task|
-				task.sleep(1)
+				sleep(1)
 			end
 			
 			expect(task.result).to be_nil
 			
-			Console.logger.debug(self) {"Stopping task..."}
 			task.stop
 			
 			expect(task.result).to be_nil
@@ -769,34 +763,34 @@ describe Async::Task do
 		end
 	end
 	
-	with '#complete?' do
-		with 'running task' do
-			it 'is not complete?' do
+	with "#complete?" do
+		with "running task" do
+			it "is not complete?" do
 				reactor.async do |task|
 					expect(task).not.to be(:complete?)
 				end
 			end
 		end
 		
-		with 'completed task' do
-			it 'is complete?' do
+		with "completed task" do
+			it "is complete?" do
 				task = reactor.async{}
 				expect(task).to be(:complete?)
 			end
 		end
 	end
 	
-	with '#stopped?' do
-		with 'running task' do
-			it 'is not stopped?' do
+	with "#stopped?" do
+		with "running task" do
+			it "is not stopped?" do
 				reactor.async do |task|
 					expect(task).not.to be(:stopped?)
 				end
 			end
 		end
 		
-		with 'stopped task' do
-			it 'is stopped?' do
+		with "stopped task" do
+			it "is stopped?" do
 				reactor.async do |task|
 					child = task.async do |task|
 						sleep(1)
@@ -810,20 +804,20 @@ describe Async::Task do
 		end
 	end
 	
-	with '#children' do
+	with "#children" do
 		it "enumerates children in same order they are created" do
 			tasks = 10.times.map do |i|
-				reactor.async(annotation: "Task #{i}") {|task| task.sleep(1)}
+				reactor.async(annotation: "Task #{i}") {sleep(1)}
 			end
 			
 			expect(reactor.children.each.to_a).to be == tasks
 		end
 	end
 	
-	with '#to_s' do
+	with "#to_s" do
 		it "should show running" do
-			apples_task = reactor.async do |task|
-				task.sleep(0.1)
+			apples_task = reactor.async do
+				sleep(0.1)
 			end
 			
 			expect(apples_task.to_s).to be =~ /running/
@@ -841,7 +835,7 @@ describe Async::Task do
 
 	it "can gets in a task" do
 		IO.pipe do |input, output|
-		  Async do
+			Async do
 				Async do
 					expect(input.gets).to be == "hello\n"
 				end
@@ -849,5 +843,146 @@ describe Async::Task do
 			end
 		end
 	end
-
+	
+	with "#defer_stop" do
+		it "can defer stopping a task" do
+			child_task = reactor.async do |task|
+				task.defer_stop do
+					sleep
+				end
+			end
+			
+			reactor.run_once(0)
+			
+			child_task.stop
+			expect(child_task).to be(:running?)
+			
+			child_task.stop
+			expect(child_task).to be(:stopped?)
+		end
+		
+		it "will stop the task if it was deferred" do
+			condition = Async::Notification.new
+			
+			child_task = reactor.async do |task|
+				task.defer_stop do
+					condition.wait
+				end
+			end
+			
+			reactor.run_once(0)
+			
+			child_task.stop(true)
+			expect(child_task).to be(:running?)
+			
+			reactor.async do
+				condition.signal
+			end
+			
+			reactor.run_once(0)
+			expect(child_task).to be(:stopped?)
+		end
+		
+		it "can defer stop in a deferred stop" do
+			child_task = reactor.async do |task|
+				task.defer_stop do
+					task.defer_stop do
+						sleep
+					end
+				end
+			end
+			
+			reactor.run_once(0)
+			
+			child_task.stop
+			expect(child_task).to be(:running?)
+			
+			child_task.stop
+			expect(child_task).to be(:stopped?)
+		end
+		
+		it "can defer stop and exit normally" do
+			child_task = reactor.async do |task|
+				task.defer_stop do
+					# Nothing.
+				end
+			end
+			
+			reactor.run_once(0)
+			
+			expect(child_task.stop_deferred?).to be == nil
+		end
+	end
+	
+	with "failing task" do
+		include_context Sus::Fixtures::Console::CapturedLogger
+		
+		it "logs a warning if a task fails without being waited on" do
+			failed_task = nil
+			
+			reactor.async do |task|
+				task.async do |task|
+					failed_task = task
+					raise "boom"
+				end
+			end
+			
+			reactor.run
+			
+			expect_console.to have_logged(
+				severity: be == :warn,
+				subject: be_equal(failed_task),
+				message: be == "Task may have ended with unhandled exception."
+			)
+		end
+		
+		it "does not log a warning if a task fails and is waited on" do
+			failed_task = nil
+			
+			reactor.async do |task|
+				expect do
+					task.async do |task|
+						# This ensures #wait is called by the parent before proceeding to raise the exception:
+						task.yield
+						failed_task = task
+						raise "boom"
+					end.wait
+				end.to raise_exception(RuntimeError, message: be =~ /boom/)
+			end
+			
+			reactor.run
+			
+			expect_console.not.to have_logged(
+				severity: be == :warn,
+			)
+		end
+	end
+	
+	with "transient task" do
+		it "is only stopped once" do
+			state = nil
+			
+			reactor.async do |task|
+				task.async(transient: true) do |task|
+					task.defer_stop do
+						state = :sleeping
+						
+						until task.stop_deferred?
+							# Bounce back to the event loop several times:
+							task.yield
+						end
+						
+						# Yield one more time, which should not cause the task to be stopped again:
+						task.yield
+						
+						state = :finished
+					end
+				end
+			end
+			
+			reactor.run
+			
+			expect(state).to be == :finished
+		end
+	end
 end
