@@ -1,10 +1,11 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2022-2023, by Samuel Williams.
+# Copyright, 2022-2024, by Samuel Williams.
+# Copyright, 2024, by Patrik Wenger.
 
-require 'async/waiter'
-require 'sus/fixtures/async'
+require "async/waiter"
+require "sus/fixtures/async"
 
 describe Async::Waiter do
 	include Sus::Fixtures::Async::ReactorContext
@@ -34,7 +35,11 @@ describe Async::Waiter do
 	end
 	
 	it "can wait for tasks even when exceptions occur" do
-		waiter.async do
+		waiter.async do |task|
+			expect(task).to receive(:warn).with_options(have_keys(
+				exception: be_a(RuntimeError),
+			)).and_return(nil)
+			
 			raise "Something went wrong"
 		end
 		
@@ -42,4 +47,14 @@ describe Async::Waiter do
 			waiter.wait
 		end.to raise_exception(RuntimeError)
 	end
+
+	with "barrier parent" do
+		let(:barrier) { Async::Barrier.new }
+		let(:waiter) { subject.new(parent: barrier) }
+
+		it "passes annotation to barrier" do
+			expect(barrier).to receive(:async).with(annotation: "waited upon task")
+			waiter.async(annotation: "waited upon task") { }
+		end
+	end
 end
