@@ -1,10 +1,10 @@
 # frozen_string_literal: true
 
 # Released under the MIT License.
-# Copyright, 2022, by Samuel Williams.
+# Copyright, 2022-2024, by Samuel Williams.
 
-require 'sus/fixtures/async'
-require 'thread'
+require "sus/fixtures/async"
+require "thread"
 
 describe Thread::Queue do	
 	include Sus::Fixtures::Async::ReactorContext
